
######################
#### bootstrap CI ####
######################

rm(list = ls())

library(readr)
library(dplyr)
library(DEoptim)
library(parallel)

# number of bootstrap replications
BB = 200
ker4 = function(x) {
  (3/2 - 1/2*x^2)*dnorm(x)
}

path = "/Users/uqfouyan/Desktop/QE replication files/Illustration/Bootstrap_CI_table_10"
setwd(path) 

# bootstrap
boot = function(B) {

# load Cracker data
data = read_csv("Cracker_t.csv")
attach(data)
N = length(unique(id))

# set.seed(B) this doesn't work for cluster computation

# sample from Cracker data on id
boot.i = sample(1:N, replace = T)
boot.data = data[id == boot.i[1],]
for (n in (2:N)) {
  boot.data = rbind(boot.data, data[id == boot.i[n],])
}

pair.boot.data = function(Data) {
  CN = combn(nrow(Data), 2); CombN = split(CN, col(CN))
  Data = as.matrix(Data)
  id = Data[,1]; t = Data[,2]; y = Data[,3]
  p1 = Data[,4]; d1 = Data[,5]; f1 = Data[,6]; p2 = Data[,7]; d2 = Data[,8]; f2 = Data[,9]
  p3 = Data[,10]; d3 = Data[,11]; f3 = Data[,12]; p4 = Data[,13]; d4 = Data[,14]; f4 = Data[,15]
  
  CombData = function(im) {
    i = im[1]; m = im[2]
    c(id[i], t[i], y[i], p1[i], d1[i], f1[i], p2[i], d2[i], f2[i], 
      p3[i], d3[i], f3[i], p4[i], d4[i], f4[i],
      id[m], t[m], y[m], p1[m], d1[m], f1[m], p2[m], d2[m], f2[m], 
      p3[m], d3[m], f3[m], p4[m], d4[m], f4[m])
  }
  
  boot.paired.data = data.frame(t(sapply(CombN, CombData))) %>%
    rename(idi = X1, ti = X2, yi = X3, pi1 = X4, di1 = X5, fi1 = X6, 
           pi2 = X7, di2 = X8, fi2 = X9, pi3 = X10, di3 = X11, fi3 = X12, 
           pi4 = X13, di4 = X14, fi4 = X15,
           idm = X16, tm = X17, ym = X18, pm1 = X19, dm1 = X20, fm1 = X21, 
           pm2 = X22, dm2 = X23, fm2 = X24,
           pm3 = X25, dm3 = X26, fm3 = X27, pm4 = X28, dm4 = X29, fm4 = X30) 
  filter(boot.paired.data, idi != idm | ti != tm)
  
}

paired.data = pair.boot.data(boot.data)
attach(paired.data)

# tuning parameter
NT = nrow(boot.data); a = 5
c1 = sd(pi1-pm1); c2 = sd(pi2-pm2); c3 = sd(pi3-pm3); c4 = sd(pi4-pm4)
hn1 = a*c1*NT^(-1/7); hn2 = a*c2*NT^(-1/7); hn3 = a*c3*NT^(-1/7); hn4 = a*c4*NT^(-1/7)

# objective function
yi1 = (yi==1); yi2 = (yi==2); yi3 = (yi==3); yi4 = (yi==4)
ym1 = (ym==1); ym2 = (ym==2); ym3 = (ym==3); ym4 = (ym==4)
  
# choice 1, match choice {2,3,4}
K1im = ker4((pi2-pm2)/hn2)*ker4((pi3-pm3)/hn3)*ker4((pi4-pm4)/hn4)*
  (di2 == dm2)*(di3 == dm3)*(di4 == dm4)*(fi2 == fm2)*(fi3 == fm3)*(fi4 == fm4)
R1im = (yi1-ym1) + (ym2-yi2) + (ym3-yi3) + (ym4-yi4)
  
# choice 2, match choice {1,3,4}  
K2im = ker4((pi1-pm1)/hn1)*ker4((pi3-pm3)/hn3)*ker4((pi4-pm4)/hn4)*
  (di1 == dm1)*(di3 == dm3)*(di4 == dm4)*(fi1 == fm1)*(fi3 == fm3)*(fi4 == fm4)
R2im = (ym1-yi1) + (yi2-ym2) + (ym3-yi3) + (ym4-yi4)
  
# choice 3, match choice {1,2,4}
K3im = ker4((pi2-pm2)/hn2)*ker4((pi1-pm1)/hn1)*ker4((pi4-pm4)/hn4)*
  (di2 == dm2)*(di1 == dm1)*(di4 == dm4)*(fi2 == fm2)*(fi1 == fm1)*(fi4 == fm4)
R3im = (ym1-yi1) + (ym2-yi2) + (yi3-ym3) + (ym4-yi4)
  
# choice 4, match choice {1,2,3}
K4im = ker4((pi2-pm2)/hn2)*ker4((pi3-pm3)/hn3)*ker4((pi1-pm1)/hn1)*
  (di2 == dm2)*(di3 == dm3)*(di1 == dm1)*(fi2 == fm2)*(fi3 == fm3)*(fi1 == fm1)
R4im = (ym1-yi1) + (ym2-yi2) + (ym3-yi3) + (yi4-ym4)
  
MRCobjN = function(b) {
  
  Ui1 = (-1)*pi1 + di1*b[1] + fi1*b[2]; Ui2 = (-1)*pi2 + di2*b[1] + fi2*b[2]
  Ui3 = (-1)*pi3 + di3*b[1] + fi3*b[2]; Ui4 = (-1)*pi4 + di4*b[1] + fi4*b[2]
  Um1 = (-1)*pm1 + dm1*b[1] + fm1*b[2]; Um2 = (-1)*pm2 + dm2*b[1] + fm2*b[2]
  Um3 = (-1)*pm3 + dm3*b[1] + fm3*b[2]; Um4 = (-1)*pm4 + dm4*b[1] + fm4*b[2]
  
  Score = K1im*R1im*sign(Ui1 - Um1) + K2im*R2im*sign(Ui2 - Um2) + 
    K3im*R3im*sign(Ui3 - Um3) + K4im*R4im*sign(Ui4 - Um4)
  2*(-1)*sum(Score)
  
}
  
# DEoptim searching for global optimum 
lb = seq(-1, -1, length = 2); ub = seq(1, 1, length = 2); maxIt = 500; tol = 1e-6
output = DEoptim(fn = MRCobjN, lower = lb, upper = ub, 
                 control = list(itermax = maxIt/2, trace = T, reltol = tol, steptol = maxIt/10))
output$optim$bestmem

}

# boot.b = matrix(unlist(sapply(1:BB, boot), use.names = F), ncol = 2, byrow = T)
boot.b = matrix(unlist(mclapply(1:BB, boot, mc.cores = 4), use.names = F), ncol = 2, byrow = T)
write.csv(boot.b, paste("boot_", BB, ".csv", sep =""), row.names = F)


