
##################
# MRC Simulation #
##################

rm(list = ls())

library(MASS)
library(parallel)
library(DEoptim)
library(mvtnorm)

path = "C:/Users/fuouy/Dropbox/Papers/my papers/KOT_2019_multinomial/simulation/continuous"
setwd(path) 

# detect the number of physical CPU cores for parallel computation
n.cores = detectCores(logical = F) 

# DGP: choice set = {0,1,2,3,4}
# Uj = xj1 + xj2*b1 + xj3*b2 - ej, j = 1,2,3,4
# U0 = 0 (outside/base choice)
J = 5                             # number of choices
K = 3                             # number of regressors
coefdgp = c(1, 1)                 # true parameters

# sample size
N = 1000
CN = combn(N, 2); CombN = split(CN, col(CN))

# bandwidth 
a = 1.0; hn = a*N^(-1/10)

# MRC simulation
MRC.simu = function(s) {
  
  # generate random sample from DGP
  set.seed(s)
  mu = seq(0, 0, length = J-1)
  sigma = matrix(0.5*seq(1, 1, length = (J-1)^2), nrow = J-1) + diag(J-1)*0.5
  e = mvrnorm(n = N, mu, sigma)
  
  x11 = rnorm(N, 0, 1); x12 = runif(N, -1, 1); x13 = runif(N, -1, 1)
  x21 = rnorm(N, 0, 1); x22 = runif(N, -1, 1); x23 = runif(N, -1, 1)
  x31 = rnorm(N, 0, 1); x32 = runif(N, -1, 1); x33 = runif(N, -1, 1)
  x41 = rnorm(N, 0, 1); x42 = runif(N, -1, 1); x43 = runif(N, -1, 1)
  
  b0 = coefdgp
  u0 = 0
  u1 = x11 + b0[1]*x12 + b0[2]*x13 - e[,1]; u2 = x21 + b0[1]*x22 + b0[2]*x23 - e[,2]
  u3 = x31 + b0[1]*x32 + b0[2]*x33 - e[,3]; u4 = x41 + b0[1]*x42 + b0[2]*x43 - e[,4]
  
  y = 0*(u1 < u0)*(u2 < u0)*(u3 < u0)*(u4 < u0) + 1*(u0 < u1)*(u2 < u1)*(u3 < u1)*(u4 < u1) + 
      2*(u0 < u2)*(u1 < u2)*(u3 < u2)*(u4 < u2) + 3*(u0 < u3)*(u1 < u3)*(u2 < u3)*(u4 < u3) +
      4*(u0 < u4)*(u1 < u4)*(u2 < u4)*(u3 < u4)
  
  # create paired data
  CombData = function(im) {
    i = im[1]; m = im[2]
    c(y[i],x11[i],x12[i],x13[i],x21[i],x22[i],x23[i],x31[i],x32[i],x33[i],x41[i],x42[i],x43[i],
      y[m],x11[m],x12[m],x13[m],x21[m],x22[m],x23[m],x31[m],x32[m],x33[m],x41[m],x42[m],x43[m])
  }
  
  paired.Data = t(sapply(CombN, CombData))
  data_i = paired.Data[,1:(ncol(paired.Data)/2)] 
  data_m = paired.Data[,(1+ncol(paired.Data)/2):(ncol(paired.Data))]
  
  yi = data_i[,1]
  x11i = data_i[,2];  x12i = data_i[,3];  x13i = data_i[,4]
  x21i = data_i[,5];  x22i = data_i[,6];  x23i = data_i[,7]
  x31i = data_i[,8];  x32i = data_i[,9];  x33i = data_i[,10]
  x41i = data_i[,11]; x42i = data_i[,12]; x43i = data_i[,13]
  
  ym = data_m[,1]
  x11m = data_m[,2];  x12m = data_m[,3];  x13m = data_m[,4]
  x21m = data_m[,5];  x22m = data_m[,6];  x23m = data_m[,7]
  x31m = data_m[,8];  x32m = data_m[,9];  x33m = data_m[,10]
  x41m = data_m[,11]; x42m = data_m[,12]; x43m = data_m[,13]
  
  # choice 1, match choices {2,3,4}
  v1im = cbind(x21i-x21m,x22i-x22m,x23i-x23m,x31i-x31m,x32i-x32m,x33i-x33m,x41i-x41m,x42i-x42m,x43i-x43m)
  K1im = dmvnorm(v1im/hn)
  R1im = ((ym==0)-(yi==0)) + ((yi==1)-(ym==1)) + ((ym==2)-(yi==2)) + ((ym==3)-(yi==3)) + ((ym==4)-(yi==4))
  
  # choice 2, match choices {1,3,4}  
  v2im = cbind(x11i-x11m,x12i-x12m,x13i-x13m,x31i-x31m,x32i-x32m,x33i-x33m,x41i-x41m,x42i-x42m,x43i-x43m)
  K2im = dmvnorm(v2im/hn)
  R2im = ((ym==0)-(yi==0)) + ((ym==1)-(yi==1)) + ((yi==2)-(ym==2)) + ((ym==3)-(yi==3)) + ((ym==4)-(yi==4))
  
  # choice 3, match choices {1,2,4}
  v3im = cbind(x11i-x11m,x12i-x12m,x13i-x13m,x21i-x21m,x22i-x22m,x23i-x23m,x41i-x41m,x42i-x42m,x43i-x43m)
  K3im = dmvnorm(v3im/hn)
  R3im = ((ym==0)-(yi==0)) + ((ym==1)-(yi==1)) + ((ym==2)-(yi==2)) + ((yi==3)-(ym==3)) + ((ym==4)-(yi==4))
  
  # choice 4, match choices {1,2,3}
  v4im = cbind(x11i-x11m,x12i-x12m,x13i-x13m,x21i-x21m,x22i-x22m,x23i-x23m,x31i-x31m,x32i-x32m,x33i-x33m)
  K4im = dmvnorm(v4im/hn)
  R4im = ((ym==0)-(yi==0)) + ((ym==1)-(yi==1)) + ((ym==2)-(yi==2)) + ((ym==3)-(yi==3)) + ((yi==4)-(ym==4))
  
  # MRC objective function 
  MRCobjN = function(b) {
    
    U1i = x11i + x12i*b[1] + x13i*b[2]; U2i = x21i + x22i*b[1] + x23i*b[2]
    U3i = x31i + x32i*b[1] + x33i*b[2]; U4i = x41i + x42i*b[1] + x43i*b[2] 
    U1m = x11m + x12m*b[1] + x13m*b[2]; U2m = x21m + x22m*b[1] + x23m*b[2] 
    U3m = x31m + x32m*b[1] + x33m*b[2]; U4m = x41m + x42m*b[1] + x43m*b[2] 
    
    Score = K1im*R1im*sign(U1i - U1m) + K2im*R2im*sign(U2i - U2m) +
            K3im*R3im*sign(U3i - U3m) + K4im*R4im*sign(U4i - U4m)
    2*(-1)*mean(Score)
  }
  
  # DEoptim
  lb = seq(0, 0, length = 2); ub = seq(2, 2, length = 2); maxIt = 1000; tol = 1e-6
  output = DEoptim(fn = MRCobjN, lower = lb, upper = ub, 
                   control = list(itermax = maxIt/5, trace = T, reltol = tol, steptol = maxIt/20))
  output$optim$bestmem
}

# number of replications
R = 1000

# Monte Carlo simulation
time.test = system.time({
  MRC.simu.par = matrix(unlist(mclapply(1:R, MRC.simu, mc.cores = n.cores), use.names = F), 
                        ncol = K-1, byrow = T)
})

# time.test = system.time({
#   MRC.simu.par = t(sapply(1:R, MRC.simu))
# })

################################
# summarize simulation results #
################################

# function summarizing simulation results
simu_results = function(output, coefdgp) {
  K = length(coefdgp)
  simu_results_k = function(k) {
    Bias = output[, k] - coefdgp[k]
    meanBias = mean(Bias)
    medBias = median(Bias)
    RMSE = sqrt(mean(Bias^2))
    MAD = median(abs(Bias - medBias))
    cbind(meanBias, RMSE, medBias, MAD)
  }
  results = t(sapply(1:K, simu_results_k))
  colnames(results) = c("MEAN", "RMSE", "MED", "MAD")
  rownames(results) = paste("par", 1:K, sep = "")
  results
}

simu.table = simu_results(MRC.simu.par, coefdgp)
time.test
simu.table
write.csv(simu.table, paste("cs3_", N, ".csv", sep =""))




