
rm(list = ls())

library(parallel)
library(MASS)
library(DEoptim)
library(mvtnorm)

path = "/Users/uqfouyan/Dropbox/Papers/my papers/KOT_2019_multinomial/simulation/continuous"
setwd(path) 

# detect the number of physical CPU cores for parallel computation
n.cores = detectCores(logical = F) 

# DGP design 5: static panel, choice set = {0,1,2}, 3 regressors, 2 time periods
# Ujt = xjt.1 + xjt.2*b1 + xjt.3*b2 + aj - ejt, j = 1,2
# U0t = 0 (outside/base choice)
J = 3                             # number of choices
K = 3                             # number of regressors
TT = 2                            # number of time periods
coefdgp = c(1, 1)                 # true parameters

# sample size
N = 10000

# bandwidth 
h1 = 3; a1 = 6*log(N)^(-1/3); h1n = a1*N^(-1/(2*h1)) 
h2 = 3; a2 = 6*log(N)^(-1/3); h2n = a2*N^(-1/(2*h2)) 

# maximum score simulation
MS.simu = function(s) {
  
  # generate random sample from DGP
  set.seed(s)
  mu = seq(0, 0, length = (J-1)*TT)
  sigma = matrix(0.5*seq(1, 1, length = ((J-1)*TT)^2), nrow = (J-1)*TT) + diag((J-1)*TT)*0.5
  e = mvrnorm(n = N, mu, sigma)
  e11 = e[,1]; e12 = e[,2]; e21 = e[,3]; e22 = e[,4] 
  
  x11.1 = rnorm(N, 0, 1); x11.2 = runif(N, -1, 1); x11.3 = runif(N, -1, 1)
  x21.1 = rnorm(N, 0, 1); x21.2 = runif(N, -1, 1); x21.3 = runif(N, -1, 1)
  x12.1 = rnorm(N, 0, 1); x12.2 = runif(N, -1, 1); x12.3 = runif(N, -1, 1)
  x22.1 = rnorm(N, 0, 1); x22.2 = runif(N, -1, 1); x22.3 = runif(N, -1, 1)
  
  a1 = (x11.1 + x12.1)/TT/2; a2 = (x21.1 + x22.1)/TT/2
  
  b0 = coefdgp
  u01 = 0
  u11 = x11.1 + b0[1]*x11.2 + b0[2]*x11.3 + a1 - e11
  u21 = x21.1 + b0[1]*x21.2 + b0[2]*x21.3 + a2 - e21
  
  u02 = 0
  u12 = x12.1 + b0[1]*x12.2 + b0[2]*x12.3 + a1 - e12
  u22 = x22.1 + b0[1]*x22.2 + b0[2]*x22.3 + a2 - e22
  
  y1 = 0*(u11 < u01)*(u21 < u01) + 1*(u01 < u11)*(u21 < u11) + 2*(u01 < u21)*(u11 < u21)
  y2 = 0*(u12 < u02)*(u22 < u02) + 1*(u02 < u12)*(u22 < u12) + 2*(u02 < u22)*(u12 < u22)
  
  # choice 1, match choice 2
  v1ts = cbind(x21.1-x22.1, x21.2-x22.2, x21.3-x22.3)
  K1ts = dmvnorm(v1ts/h1n)
  R1ts = ((y1==1)-(y2==1)) + ((y2==2)-(y1==2)) + ((y2==0)-(y1==0)) 
  
  # choice 2, match choice 1  
  v2ts = cbind(x11.1-x12.1, x11.2-x12.2, x11.3-x12.3)
  K2ts = dmvnorm(v2ts/h1n)
  R2ts = ((y2==1)-(y1==1)) + ((y1==2)-(y2==2)) + ((y2==0)-(y1==0)) 
  
  # maximum score objective function 
  MSobjN = function(b) {
    
    U11 = x11.1 + x11.2*b[1] + x11.3*b[2]; U21 = x21.1 + x21.2*b[1] + x21.3*b[2]
    U12 = x12.1 + x12.2*b[1] + x12.3*b[2]; U22 = x22.1 + x22.2*b[1] + x22.3*b[2]
    
    Score = K1ts*R1ts*sign(U11 - U12) + K2ts*R2ts*sign(U21 - U22)
    (-1)*sum(Score)
    
  }
  
  # DEoptim stage 1
  lb = c(0, 0); ub = c(2, 2); maxIt = 1000; tol = 1e-6
  output = DEoptim(fn = MSobjN, lower = lb, upper = ub, 
                   control = list(itermax = maxIt/5, trace = T, reltol = tol, steptol = maxIt/10))
  bhat = output$optim$bestmem
  
  ## DEoptim stage 2
  V11 = x11.1 + bhat[1]*x11.2 + bhat[2]*x11.3; V21 = x21.1 + bhat[1]*x21.2 + bhat[2]*x21.3 
  V12 = x12.1 + bhat[1]*x12.2 + bhat[2]*x12.3; V22 = x22.1 + bhat[1]*x22.2 + bhat[2]*x22.3 
  C1ts = dnorm((V21-V22)/h2n)
  C2ts = dnorm((V11-V12)/h2n)
  
  MSobjN2 = function(b) {
    
    U11 = x11.1 + x11.2*b[1] + x11.3*b[2]; U21 = x21.1 + x21.2*b[1] + x21.3*b[2]
    U12 = x12.1 + x12.2*b[1] + x12.3*b[2]; U22 = x22.1 + x22.2*b[1] + x22.3*b[2]
    
    Score = C1ts*R1ts*sign(U11 - U12) + C2ts*R2ts*sign(U21 - U22)
    (-1)*sum(Score)
    
  }
  
  output2 = DEoptim(fn = MSobjN2, lower = lb, upper = ub, 
                     control = list(itermax = maxIt/5, trace = T, reltol = tol))
  output2$optim$bestmem
  
}

# number of replications
R = 1000

# Monte-Carlo simulation
time.test = system.time({
  MS.simu.par = matrix(unlist(mclapply(1:R, MS.simu, mc.cores = n.cores-2), use.names = F), 
                       ncol = K-1, byrow = T)
})

################################
# summarize simulation results #
################################

# function summarizing simulation results
simu_results = function(output, coefdgp) {
  K = length(coefdgp)
  simu_results_k = function(k) {
    Bias = output[, k] - coefdgp[k]
    meanBias = mean(Bias)
    medBias = median(Bias)
    RMSE = sqrt(mean(Bias^2))
    MAD = median(abs(Bias - medBias))
    cbind(meanBias, RMSE, medBias, MAD)
  }
  results = t(sapply(1:K, simu_results_k))
  colnames(results) = c("MEAN", "RMSE", "MED", "MAD")
  rownames(results) = paste("par", 1:K, sep = "")
  results
}

simu.table = simu_results(MS.simu.par, coefdgp)
time.test
simu.table
write.csv(simu.table, paste("sp_", N, ".csv", sep =""))



