
rm(list = ls())

library(parallel)
library(MASS)
library(DEoptim)
library(mvtnorm)

path = "/Users/uqfouyan/Dropbox/Papers/my papers/KOT_2019_multinomial/simulation/continuous"
setwd(path) 

# detect the number of physical CPU cores for parallel computation
n.cores = detectCores(logical = F) 

# DGP design 5: dynamic panel, choice set = {0,1,2}, 4 regressors (including lagged y), 4 time periods
# U1t = x1t.1 + x1t.2*b + (yt-1 = j)*r + a1 - e1t
# U2t = x2t.1 + x2t.2*b + a2 - e2t
# U0t = 0 (outside/base choice)
J = 3                             # number of choices
K = 3                             # number of regressors
TT = 4                            # number of time periods
coefdgp = c(1, 1/2)               # true parameters

# sample size 
NN = c(500, 1000, 2000, 5000, 10000)

for (N in NN) {

# bandwidth 
h1 = 6; a1 = 2*log(N)^(-1/4); hn1 = a1*N^(-1/(2*h1)) 
h2 = 2; a2 = 6*log(N)^(-1/3); hn2 = a2*N^(-1/(2*h2)) 

############################
# maximum score simulation #
############################

MS.simu = function(s) {
  
  #####################################
  ## generate random sample from DGP ##
  #####################################
  
  set.seed(s)
  mu = seq(0, 0, length = (J-1))
  sigma = matrix(0.5*seq(1, 1, length = (J-1)^2), nrow = (J-1)) + diag((J-1))*0.5
  e0 = mvrnorm(n = N, mu, sigma); e10 = e0[,1]; e20 = e0[,2]
  e1 = mvrnorm(n = N, mu, sigma); e11 = e1[,1]; e21 = e1[,2]
  e2 = mvrnorm(n = N, mu, sigma); e12 = e2[,1]; e22 = e2[,2]
  e3 = mvrnorm(n = N, mu, sigma); e13 = e3[,1]; e23 = e3[,2]
  
  x10.1 = rnorm(N, 0, 1); x10.2 = runif(N, -1, 1)
  x20.1 = runif(N, -1, 1); x20.2 = runif(N, -1, 1)
  
  x11.1 = rnorm(N, 0, 1); x11.2 = runif(N, -1, 1)
  x21.1 = runif(N, -1, 1); x21.2 = runif(N, -1, 1)
  
  x12.1 = rnorm(N, 0, 1); x12.2 = runif(N, -1, 1)
  x22.1 = runif(N, -1, 1); x22.2 = runif(N, -1, 1)
  
  x13.1 = rnorm(N, 0, 1); x13.2 = runif(N, -1, 1)
  x23.1 = runif(N, -1, 1); x23.2 = runif(N, -1, 1)
  
  a1 = (x10.1 + x11.1 + x12.1 + x13.1)/TT/2; a2 = (x20.1 + x21.1 + x22.1 + x23.1)/TT/2
  
  b0 = coefdgp
  
  u00 = 0
  u10 = x10.1 + b0[1]*x10.2 + a1 - e10
  u20 = x20.1 + b0[1]*x20.2 + a2 - e20
  y0 = 0*(u10 < u00)*(u20 < u00) + 1*(u00 < u10)*(u20 < u10) + 2*(u00 < u20)*(u10 < u20)
  
  u01 = 0
  u11 = x11.1 + b0[1]*x11.2 + b0[2]*(y0 == 1) + a1 - e11
  u21 = x21.1 + b0[1]*x21.2 + a2 - e21
  y1 = 0*(u11 < u01)*(u21 < u01) + 1*(u01 < u11)*(u21 < u11) + 2*(u01 < u21)*(u11 < u21)
  
  u02 = 0
  u12 = x12.1 + b0[1]*x12.2 + b0[2]*(y1 == 1) + a1 - e12
  u22 = x22.1 + b0[1]*x22.2 + a2 - e22
  y2 = 0*(u12 < u02)*(u22 < u02) + 1*(u02 < u12)*(u22 < u12) + 2*(u02 < u22)*(u12 < u22)
  
  u03 = 0
  u13 = x13.1 + b0[1]*x13.2 + b0[2]*(y2 == 1) + a1 - e13
  u23 = x23.1 + b0[1]*x23.2 + a2 - e23
  y3 = 0*(u13 < u03)*(u23 < u03) + 1*(u03 < u13)*(u23 < u13) + 2*(u03 < u23)*(u13 < u23)
  
  ##############
  ## choice 1 ##
  ##############
  
  # match all x in t = 2 and t = 3
  K1 = dmvnorm(cbind(x12.1-x13.1, x12.2-x13.2, x22.1-x23.1, x22.2-x23.2, x21.1-x22.1, x21.2-x22.2)/hn1)
  
  # observed choices 
  d10 = (y0 == 1); d13 = (y3 == 1); AB = (y1==1) - (y2==1)
  
  ##############
  ## choice 2 ##
  ##############
  
  # match choice 1 in t = 1 and t = 2
  K2.12 = dmvnorm(cbind(x11.1-x12.1, x11.2-x12.2)/hn2)*((y0 == 1) == (y1 == 1))
  R2.12 = ((y2==1) - (y1==1)) + ((y1==2) - (y2==2)) + ((y2==0) - (y1==0))
  
  # match choice 1 in t = 1 and t = 3
  K2.13 = dmvnorm(cbind(x11.1-x13.1, x11.2-x13.2)/hn2)*((y0 == 1) == (y2 == 1))
  R2.13 = ((y3==1) - (y1==1)) + ((y1==2) - (y3==2)) + ((y3==0) - (y1==0))
  
  # match choice 1 in t = 2 and t = 3
  K2.23 = dmvnorm(cbind(x12.1-x13.1, x12.2-x13.2)/hn2)*((y1 == 1) == (y2 == 1))
  R2.23 = ((y3==1) - (y2==1)) + ((y2==2) - (y3==2)) + ((y3==0) - (y2==0))
  
  ######################################
  ## maximum score objective function ##
  ######################################
  
  MSobjN = function(coef) {
    
    b = coef[1]; r = coef[2]
    
    # utility of choice 1 in t = 1 and t = 2
    U1.12 = sign(x11.1 + b*x11.2 + r*d10 - x12.1 - b*x12.2 - r*d13)
    
    # utility of choice 2
    U2.12 = sign(x21.1 + b*x21.2 - (x22.1 + b*x22.2))
    U2.13 = sign(x21.1 + b*x21.2 - (x23.1 + b*x23.2))
    U2.23 = sign(x22.1 + b*x22.2 - (x23.1 + b*x23.2))
    
    (-1)*sum(K1*AB*U1.12 + K2.12*R2.12*U2.12 + K2.13*R2.13*U2.13 + K2.23*R2.23*U2.23)
    
  }
  
  #############
  ## DEoptim ##
  #############
  
  lb = c(0, 0); ub = c(2, 1); maxIt = 1000; tol = 1e-6
  output = DEoptim(fn = MSobjN, lower = lb, upper = ub, 
                   control = list(itermax = maxIt/5, trace = T, reltol = tol, steptol = maxIt/10))
  output$optim$bestmem
  
}

# number of replications
R = 1000

# Monte-Carlo simulation
time.test = system.time({
  MS.simu.par = matrix(unlist(mclapply(1:R, MS.simu, mc.cores = n.cores), use.names = F), 
                       ncol = K-1, byrow = T)
})

################################
# summarize simulation results #
################################

# function summarizing simulation results
simu_results = function(output, coefdgp) {
  K = length(coefdgp)
  simu_results_k = function(k) {
    Bias = output[, k] - coefdgp[k]
    meanBias = mean(Bias)
    medBias = median(Bias)
    RMSE = sqrt(mean(Bias^2))
    MAD = median(abs(Bias - medBias))
    cbind(meanBias, RMSE, medBias, MAD)
  }
  results = t(sapply(1:K, simu_results_k))
  colnames(results) = c("MEAN", "RMSE", "MED", "MAD")
  rownames(results) = paste("par", 1:K, sep = "")
  results
}

simu.table = simu_results(MS.simu.par, coefdgp)
time.test
simu.table
write.csv(simu.table, paste("dp_", N, ".csv", sep =""))

}

